----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:17:56 01/25/2018 
-- Design Name: 
-- Module Name:    A2_SD_TOP - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity A2_SD_TOP is
    Port ( 
			  -- Standard Apple ][ slot signals we use
			  cpu_addr 		: in  STD_LOGIC_VECTOR (15 downto 0);
           cpu_data 		: inout  STD_LOGIC_VECTOR (7 downto 0);
           n_iosel 		: in  STD_LOGIC;
           n_devsel 		: in  STD_LOGIC;
           n_iostrobe 	: in  STD_LOGIC;
           r_nw 			: in  STD_LOGIC;
           n_reset 		: in  STD_LOGIC;
           n_irq 			: out  STD_LOGIC;
           n_nmi 			: out  STD_LOGIC;
           clk_7m 		: in  STD_LOGIC;
           clk_ph0 		: in  STD_LOGIC;
           clk_ph1 		: in  STD_LOGIC;
			  
			  -- These might be used later
			  flash_addr	: OUT	STD_LOGIC_VECTOR (18 downto 12);
			  sd_detect		: IN 	STD_LOGIC;
			  sd_wprot		: IN	STD_LOGIC;
			  
			  -- Control lines for the I/O buffers to the Apple ][ slot
           addr_en 		: out  STD_LOGIC;
           data_en 		: out  STD_LOGIC;
			  
			  -- Control signal to enable writing to EEPROM
			  jmp_we			: in std_logic;
			  
			  -- Control signal(s) for the EEPROM
			  ROM_n_oe		: out std_logic;
			  ROM_n_ce		: out std_logic;
			  ROM_n_we		: out std_logic;
			  
			  -- Control signals for the SPI environment we are going to
			  -- use for the SD card
			  spi_int		: in std_logic_vector(0 to 3);
			  spi_miso		: in std_logic_vector(0 to 3);
			  spi_mosi		: out std_logic;
			  spi_n_sel		: out std_logic_vector(0 to 3);
			  spi_sclk		: out std_logic
			);
end A2_SD_TOP;

architecture Behavioral of A2_SD_TOP is

	signal n_cfff			: std_logic;
	signal clk_ph2			: std_logic;
	signal clk_n_phi2			: std_logic;
	signal cs1_6502		: std_logic;
	signal n_cs2_6502		: std_logic;
	--signal n_res			: std_logic;
	signal diag				: std_logic;
	
begin

-- This is the (unmodified) SPI6502B module
-- Created and publisched by Andre Fachat (6502.org)
-- TODO: Add proper info!
SPI6502B: entity work.SPI6502B 
	PORT MAP(
		cpu_d 				=> cpu_data,
		cpu_rnw 				=> r_nw,
		cpu_Nirq 			=> n_irq,
		cpu_Nres 			=> n_reset,
		cpu_a 				=> cpu_addr(1 downto 0),
		cpu_Nphi2		 	=> clk_n_phi2,
		cs1 					=> cs1_6502,
		Ncs2 					=> n_cs2_6502,
		extclk 				=> clk_7m,
		spi_miso 			=> spi_miso,
		spi_mosi 			=> spi_mosi,
		spi_sclk 			=> spi_sclk,
		spi_Nsel 			=> spi_n_sel,
		spi_int 				=> spi_int,
		diag 					=> diag
	);
	
-- This is used to generate the /CFFF signal, so we can switch in the
-- card AUX ROM space ($C800-$CFFF)
AUXROM_en : entity work.AUXROM_en
	port map (
		n_iostrobe			=> n_iostrobe,
		cpu_addr 			=> cpu_addr(10 downto 0),          
		n_cfff 				=> n_cfff
	);

-- Generate the PH2 clock signal, we are (still) missing in the Apple ][
-- slot(s)
GEN_CLK_PHI2 : entity work.clk_gen_phi2 
	PORT MAP(
		clk_phi0 			=> clk_ph0, 
		clk_7m 				=> clk_7m, 
		clk_phi2		 		=> clk_ph2
   );
	
-- Create the needed inverted PH2 clock
clk_n_phi2 <= not clk_ph2;

-- Create the data_en signal using the n_devsel, n_iosel and n_iostrobe signals
data_en <= not ((not n_devsel) or (not n_iosel) or (not n_iostrobe));

-- EEPROM /OE signal generation
EEPROM_n_OE : entity work.EPROM_en 
	PORT MAP(
		n_cfff 				=> n_cfff, 
		n_iosel 				=> n_iosel, 
		r_nw 					=> r_nw, 
		ROM_oe 				=> ROM_n_oe
   );
	
-- EEPROM /CE signal generation
EEPROM_n_CE : entity work.EEPROM_CE 
	PORT MAP(
		EEPROM_n_CE => rom_n_ce, 
		n_iosel => n_iosel, 
		n_iostrobe => n_iostrobe
   );	
	
-- EEPROM /WE signal generation
EEPROM_n_WE : entity work.EEPROM_n_WE 
	PORT MAP(
		EEPROM_n_WE => rom_n_we, 
		crd_WE => jmp_we, 
		r_nw => r_nw
   );
end Behavioral;

